# conformedl.sh

# Parse the command line arguments...

GOOD_ARGS=0
CASE_SENSITIVE=1
MATCH_NAMES=0
COUNT=0
DELETE_UNWANTED_FRAMES=0
DISKSPACE_CHECK=1
HANDLES=0
TEST=0

if [ $# -ge 2 ]; then

    if [ $# -le 9 ]; then

        # EDL name and frame rate must be provided
        EDL_NAME=$1
        FRAME_RATE=$2

        GOOD_ARGS=1

        # check for optional arguments
        HANDLE_VALUE_ARGUMENT=0
        for i in $@
        do
            let COUNT=$COUNT+1
            if [ $COUNT -gt 2 ]; then
                if [ $HANDLE_VALUE_ARGUMENT -eq 1 ]; then
                    HANDLES=$i
                    HANDLE_VALUE_ARGUMENT=0
                elif [ $i = "-i" ]; then
                    CASE_SENSITIVE=0
                elif [ $i = "-n" ]; then
                    MATCH_NAMES=1
                elif [ $i = "-d" ]; then
                    DELETE_UNWANTED_FRAMES=1
                elif [ $i = "-f" ]; then
                    DISKSPACE_CHECK=0
                elif [ $i = "-h" ]; then
                    HANDLE_VALUE_ARGUMENT=1
                elif [ $i = "-t" ]; then
                    TEST=1
                else
                    GOOD_ARGS=0
                fi
            fi
        done

    fi
fi

if [ $GOOD_ARGS -eq 0 ]; then
    echo "conformedl.sh <edl_name> <frame_rate> [-i] [-n] [-d] [-f] [-h <frames>]"
    echo "where:"
    echo "    -d = delete frames unarchived not required (e.g. from dpx/ari file chunks)"
    echo "    -f = force unarchive (don't insist on 5GB free to unarchive to system disk)"
    echo "    -h = handles (in frames) on start and end points when unarchived"
    echo "    -i = roll and/or shot name is case insensitive (default case sensitive)"
    echo "    -n = match on shot names in EDL comments rather than timecodes"
    echo "    -t = test - do not execute getting frames from disk just display hits"
    exit -1
fi

# if handles are specified then we must delete unwanted frames too - regardless
# of whether its specified in the command line
if [ $HANDLES -gt 0 ]; then
    if [ $DELETE_UNWANTED_FRAMES -eq 0 ]; then
        DELETE_UNWANTED_FRAMES=1
    fi
fi

echo ===============================================================================
echo Codex Digital EDL conform from tape
echo 

# remove old copies of files if they exist
rm -f failededl.sh
rm -f failedtape.sh
rm -f eventsdone.lst
rm -f dupes.lst
rm -f foundevents.lst
rm -f allevents.lst

# check EDL exists
if [ ! -f $EDL_NAME ]; then
    echo File $EDL_NAME does not exists.  Aborted conform.
    exit -1
fi
NUM_LINES=`cat $EDL_NAME | wc -l`
if [ $NUM_LINES -eq 0 ]; then
    echo "$EDL_NAME has 0 lines - possible OS line ending issue."
    cp $EDL_NAME $EDL_NAME.dos
    dos2unix $EDL_NAME.dos
    NUM_LINES=`cat $EDL_NAME.dos | wc -l`
    echo "Windows version has $NUM_LINES lines"
    if [ $NUM_LINES -eq 0 ]; then
        rm -f $EDL_NAME.dos
        echo "Converting from Mac to Linux"
        tr '\r' '\n' < $EDL_NAME > $EDL_NAME.mac
        NUM_LINES=`cat $EDL_NAME.mac | wc -l`
        echo "Mac version has $NUM_LINES lines"
        if [ $NUM_LINES -eq 0 ]; then
            rm -f $EDL_NAME.mac
            echo "No valid lines found in $EDL_NAME"
            echo "Possibly corrupt?"
            exit - 1
        else
            mv $EDL_NAME.mac $EDL_NAME 
        fi
    else
        mv $EDL_NAME.dos $EDL_NAME
    fi
    echo
fi 

# Create file with list of all events in EDL
awk '{ if ((NF >= 6) && ($1 ~ /^[0-9][0-9]*$/)) print $1; }' $EDL_NAME > allevents.lst
COUNT=0
for i in `cat ./allevents.lst`;
do
    EVENTLIST[$COUNT]=$i
    let COUNT=$COUNT+1
done

# Create script xedlsearch.sh which contains a list of findtape calls 
# for each entry in the edl.  The script also puts a copy of any calls
# that failed into failededl.sh
echo Creating search scripts...
awk -v numEvents=$COUNT -v fps=$FRAME_RATE -v matchNames=$MATCH_NAMES -v caseSense=$CASE_SENSITIVE -v handles=$HANDLES ' 
    (NR == 1) {
       lastroll="";
       lastctx="";
       eventCount=0;
    }

    function strip_whitespace(in_string)
    {
        out_string = in_string;
        sub(/^[ \t\n\r]+/, "", out_string);
        sub(/[ \t\n\r]+$/, "", out_string);
        return out_string;
    }

    function constructCommand(rollname, context, specifics)
    {
        script = "/home/codex/findtape.sh";
        root = sprintf("-v fps=\"%d\" -v handle=\"%d\" -v rollname=\"%s\"", 
                       fps, handles, rollname);
        if (caseSense)
            ignore = 0;
        else
            ignore = 1;
        case = sprintf("-v ignorecase=\"%d\"", ignore);
        stub = script " " root " " case " " specifics;
        if (stub in prevCmds)
        {
            printf("echo Event %s already covered by Event %s\n", 
                   context, prevCmds[stub]);
        }
        else
        {
            prevCmds[stub] = context;
            ctx = sprintf("-v context=\"%s\"", context);
            cmd = stub " " ctx;
            eventCount++;
            printf("echo \"Searching for match on %s of %s (Event number %s)\" 1>&2; ",
                   eventCount, numEvents, context);
            outputCommand(cmd);
        }
        return cmd;
    }

    function outputCommand(cmd)
    {
        # output the command (which is piped into xedlsearch.sh)
        # writing the command to the failed script if the error
        # code returned indicates it failed
        printf("if ! %s; then echo \"%s\" >> failededl.sh; fi\n", 
               cmd, cmd);     
    }

    # if this line in the edl has more than 6 fields in it
    # then it must be an edl event
    ((NF >= 6) && ($1 ~ /^[0-9][0-9]*$/)) {
        if ($1 != "*")
        {
            lastroll = $2;
            lastctx = $1;

            if (matchNames == "0")
            {
                # create the command with relevant arguments
                # n.b. we include the event number as context so scripts
                # can output useful debug that refers back to the edl
                specifics = sprintf("-v timecode1=\"%s\" -v timecode2=\"%s\"", 
                                    $5, $6); 
                cmd = constructCommand($2, $1, specifics);
            }
        }
    }

    /FROM CLIP NAME/ {
        if (matchNames == "1")
        {
            shotname = strip_whitespace($5);
            specifics = sprintf("-v shotname=\"%s\"", shotname);
            cmd = constructCommand(lastroll, lastctx, specifics);
        }
    }

' $EDL_NAME > xedlsearch.sh

# make sure script is executable
chmod 755 xedlsearch.sh

# The number of events in the EDL should be equal to the number of lines
# in the script.
NUM_EDL_EVENTS=`cat xedlsearch.sh | wc -l`
printf "EDL %s contained %d event(s)\n" $EDL_NAME $NUM_EDL_EVENTS
if [ $CASE_SENSITIVE -eq 0 ]; then
    echo "(Case insensitive matching on roll and/or shot names)"
fi
if [ $MATCH_NAMES -eq 1 ]; then
    echo "(Matching EDL to archive on shot names NOT timecodes)"
fi
printf "\n"

if [ $NUM_EDL_EVENTS -eq 0 ]; then
    echo ===============================================================================
    echo Nothing to do. 
    echo Possibly a bad file or a parse error...
    if [ $CASE_SENSITIVE -eq 1 ]; then
        echo Or case senstitive mismatch...
    fi
    if [ $MATCH_NAMES -eq 1 ]; then
        echo Or failed to match names in EDL comments to archives.
    else
        echo Or failed to find timecodes in archives.
    fi
    exit 0
fi

# Invoke the findtape calls and filter the output to get the tape and 
# filemarks where the dpx's for each event live.
# These values are then formatted to produce another script (xgrab.sh)
# which produces a list of getframes.sh calls which actually copy the
# the dpx's off tape.
echo Executing searching scripts for each found event...
./xedlsearch.sh | 
    awk -v deleteExtras=$DELETE_UNWANTED_FRAMES 'BEGIN{eventCount=0;}

    # search for line output from findtape with details on where frames
    # for edl event (timecode/roll combination) are

    /^Event/ {
        # notification of a duplicate event in the form
        # "Event %s already covered by Event %s"
        if (NF == 7)
        {
            event = $2;
            printf("Event %s already covered by Event %s\n", $2, $7);
        }
    }
    /^TOC/ {
        newToc = 1;
    }
    /^Tape:/ { 
        if ($4)
        {
            # work out edl event number from last field if present
            endField = NF;
            eventFound = 0;
            if ($endField && 
                 substr($endField, 1, 1) == "(" &&
                 substr($endField, length($endField), 1) == ")")
            {
                event = $NF;

                event = substr(event, 2, length($endField)-2);
                endField = endField - 1;
                eventFound = 1;
            }

            # Note if the same event matches to material on multiple tapes
            # in multiple offloads ALL will be unarchived at the moment.
            # This logic may want to be alter to only allow sections on tapes 
            # in the first offload that has a match for the event (to prevent 
            # duplication in unarchiving same stuff from multiple offloads)
            # Conversley, if a shots files were spread over more than one offload
            # this would then not work properly.
            newEvent = eventFound;
            for (i=1; i<=eventCount && eventFound && newEvent; i++)
            {
                if (eventList[i] == event)
                {
                    lastIdx = index(lastInputLine, "marks:");
                    lastEntrySections = substr(lastInputLine, lastIdx);
                    thisIdx = index($0, "marks:");
                    thisEntrySections = substr($0, thisIdx);
                    if (thisEntrySections == lastEntrySections)
                    {
                        print "00000000a DUPE TAPE: "$2" = "lastTape;
                        newEvent = 0;
                    }
                }
            }
            if (newEvent)
                eventList[++eventCount] = event; 

            tapename = $2;

            # there may be more than 1 continuous group of filemarks
            # that need to be read to get the frames of interest - so
            # there is a getframes call for each section
            for (i=4; newEvent && i<=endField; i++)
            {
                mark = $i

                # if this section is more than one filemark then there
                # will be dash between the start and end filemarks
                spanIdx = index(mark, "-");
                if (spanIdx)
                {
                    split(mark, marks, "-");
                    first = marks[1];
                    last = marks[2];
                }
                else
                {
                    first = strtonum(mark);
                    last = first;
                }

                startOffset = 0;
                startOffIdx = index(first, "begin:");
                if (startOffIdx && (deleteExtras+0))
                {
                    startOffIdx += 6;
                    startOffset = strtonum(substr(first, startOffIdx, 
                                           length(first) - startOffIdx));
                }

                endOffset = 0;
                endOffIdx = index(last, "end:");
                if (endOffIdx && deleteExtras)
                {
                    endOffIdx += 4;
                    endOffset = strtonum(substr(last, endOffIdx,
                                         length(last) - endOffIdx));
                }

                if (first == last)
                    section = sprintf("%04d", strtonum(first))
                else
                    section = sprintf("%04d %04d", 
                                   strtonum(first), strtonum(last));

                if (startOffset)
                    section = section " -s " startOffset;
                if (endOffset)
                    section = section " -e " endOffset;

                line = tapename " " section " " event;

                print line;
            }
            lastInputLine = $0;
            lastTape = $2;
        }
    }' > tapesections.lst

# sort
awk '{ print $0 | "sort" }' tapesections.lst > tapesections.srt
rm -f tapesections.lst

# remove duplicates
awk '! seen[$0]++' tapesections.srt > tapesections.tdy
rm -f tapesection.srt

# find contiguous sections and amalgamate
awk -f /home/codex/tapemerge.awk tapesections.tdy > tapesections.mrg
rm -f tapesections.tdy

# build commands from merged tape section list
awk -v diskspace=$DISKSPACE_CHECK '

    /^Event / {
        # this line informs us an event is a duplicate in the form:
        # "Event %s already covered by Event %s" in tapesections.lst etc
        dupecount++;
        duplicate[dupecount] = $2;
        original[dupecount] = $7;
        next;
    }

    (NF >= 3) {
        event = substr($NF, 2, length($NF)-2);
        num_events = split(event, events, ",");

        cmd = "/home/codex/getframes.sh";
        for (i=1; i<NF; i++)
        {
            if ($i ~ /^[0-9]+$/)
            {
                tmp = sprintf("%d", $i + 0);
                cmd = cmd " " tmp;
            }
            else
                cmd = cmd " " $i;
        }
            
        if (!diskspace)
           cmd = cmd " -f";

        # output the script
        printf("%s\n", cmd);
        printf("ret=$?\n");
        printf("if [ $ret -eq 254 ]; then\n");
        printf("{\n");
        printf("    echo Aborted.\n");
        printf("    exit -1\n");
        printf("}\n");
        printf("elif [ $ret -eq 255 ]; then\n");
        printf("{\n");
        if (length(event))
        {
            printf("\techo \"# Failed to get frames for EDL event(s) %s\" >> failedtape.sh\n", event);
            printf("\techo \"%s\" >> failedtape.sh\n", cmd);
           
            if (num_events > 1)
            {
                for (i=1; i<=num_events; i++)
                    printf("\techo \"Failed to get frames for EDL event %s\"\n", events[i]);
            }
            else
                printf("\techo \"Failed to get frames for EDL event %s\"\n", event);
            printf("}\n");
            printf("else\n");
            printf("{\n");
            if (num_events > 1)
            {
                for (i=1; i<=num_events; i++)
                {
                    printf("\techo \"Got frames for EDL event %s OK\"\n", events[i]);
                    printf("\techo \"Got frames for EDL event %s OK\" >> eventsdone.lst\n", events[i]);
                }
            }
            else
            {
                printf("\techo \"Got frames for EDL event %s OK\"\n", event);
                printf("\techo \"Got frames for EDL event %s OK\" >> eventsdone.lst\n", event);
            }
            printf("\techo\n");
        }
        else
            printf("\techo \"%s\" >> failedtape.sh\n", cmd);
        printf("}\n");
        printf("fi\n");
    }

    END {
        for (i=1; i<=dupecount; i++)
        {
            printf("echo \"Got frames for EDL event %s (assuming Event %s unarchived OK)\" >> dupes.lst\n", 
                   duplicate[i], original[i]);
        }
        
    }

' tapesections.mrg > xgrab.sh

rm -f tapesections.mrg

# make sure the script we've produced is executable
chmod 755 xgrab.sh

# cronky way of working out how many events from the edl we have found
# dpx's on tape for
grep "Got frames for EDL event" xgrab.sh > foundevents.lst
NUM_EDL_EVENTS_FOUND=`cat foundevents.lst | wc -l`
if [ $NUM_EDL_EVENTS -gt $NUM_EDL_EVENTS_FOUND ]; then
   NUM_FAILED=$[$NUM_EDL_EVENTS-$NUM_EDL_EVENTS_FOUND]
   echo "$NUM_FAILED EDL event(s) could not be found on any tapes"
   echo

   if [ $NUM_EDL_EVENTS_FOUND -eq 0 ]; then
      echo ===============================================================================
      echo None of the EDL events match any shots in any archive known to this system.
      printf "Check:\n"
      printf "1. Timecode and rolls match between edl and shot.\n"
      printf "2. Rolls and shotnames and use case insensitive flag if needed.\n"
      printf "3. Table of contents files (TOCs) are present and in the correct place.\n"
      echo TOTAL FAILURE
      exit -1
   fi

   echo "Unarchiving material for matched EDL event(s)"
   echo
fi
rm -f foundevents.lst

if [ $TEST -eq 1 ]; then
   echo "testing: examine xgrab.sh for results"
   exit -1
fi

# execute the script
./xgrab.sh
echo

# if there were any entries in the edl that we couldn't find matches
# for on tape then failededl.sh will exist and have the calls to 
# findtape that failed from xedlsearch.sh in it
if [ -s failededl.sh ]; then
    chmod 755 failededl.sh
    echo "The following findtape.sh calls from edl lines failed:"
    cat failededl.sh
    echo "These can be reattempted by executing failededl.sh"
    echo
fi

# Of the edl events that we did find dpxs on tape, if the attempt to get
# the dpxs failed then they will be in failedtape.sh
if [ -s failedtape.sh ]; then
    chmod 755 failedtape.sh
    echo "The following getframes.sh calls failed:"
    cat failedtape.sh
    echo "These can be reattempted by executing failedtape.sh"
    echo
fi

echo ===============================================================================
echo EDL event unarchiving results:
SUCCESS_COUNT=0
for (( i=0; i<$COUNT; i++ ))
do
    EVENT_STRING=${EVENTLIST[$i]}

    # get rid of leading zeroes
    EVENT_NUMBER=`echo $EVENT_STRING | awk '{print $1 + 0}'`

    NOT_ON_TAPES=0
    if [ -f failededl.sh ]; then
        if grep "context=$EVENT_STRING" failededl.sh > /dev/nul; then 
            NOT_ON_TAPES=1
        fi
    fi
    
    FAILED_GET_FROM_TAPES=0
    if [ -f failedtape.sh ]; then
        if grep -w "EDL event.*$EVENT_NUMBER" failedtape.sh > /dev/nul; then 
            FAILED_GET_FROM_TAPES=1
        fi
    fi

    DUPE=0
    if [ -f dupes.lst ]; then
        if grep -w "EDL event $EVENT_STRING" dupes.lst > /dev/nul; then 
            ORIGINAL=`awk -v event=$EVENT_NUMBER '{ if (($6+0) == event) print $9+0; }' dupes.lst`
            DUPE=1
        fi
    fi

    if [ $NOT_ON_TAPES -eq 1 ]; then 
        echo "$EVENT_NUMBER - ERROR - not found on tapes"
    elif [ $FAILED_GET_FROM_TAPES -eq 1 ]; then 
        echo "$EVENT_NUMBER - ERROR - unarchiving from tape failed"
    elif [ $DUPE -eq 1 ] && [ ${SUCCESS_ARRAY[$ORIGINAL]} ]; then
        echo "$EVENT_NUMBER - SUCCESS - dupe of event $ORIGINAL"
        let SUCCESS_COUNT=$SUCCESS_COUNT+1
    elif [ $DUPE -eq 1 ] && [ ! ${SUCCESS_ARRAY[$ORIGINAL]} ]; then
        echo "$EVENT_NUMBER - ERROR - dupe of failed event $ORIGINAL"
    else 
        echo "$EVENT_NUMBER - SUCCESS"; 
        let SUCCESS_COUNT=$SUCCESS_COUNT+1
        SUCCESS_ARRAY[$EVENT_NUMBER]=1
    fi
done
if [ $SUCCESS_COUNT -eq $COUNT ]; then
    echo TOTALLY SUCCESSFUL
elif [ $SUCCESS_COUNT -ne 0 ]; then
    echo PARTIALLY SUCCESSFUL
    echo "(see above and failededl.sh and failedtape.sh for details)"
else
    echo TOTAL FAILURE!
    echo "(see above and failededl.sh and failedtape.sh for details)"
fi

# tidy up
rm -f xgrab.sh
rm -f xedlsearch.sh
rm -f allevents.lst

