#!/bin/sh

if [ ! -f ./tape_list.log ]; then
    echo "# Used tape list" >> ./tape_list.log
fi

echo rewinding...
mt -f /dev/nst0 rewind
echo setblk...
mt -f /dev/nst0 setblk 0

not_end_of_tape ()
{
    res=`mt -f /dev/nst0 status`
    if grep -q EOD <<<$res
    then 
        echo EOD
        return 1;
    else
        return 0;
    fi
}

OLD_FILES=`ls`
START_SECONDS=`date +%s`
#du can take a long time on a san so disable this (used for bandwidth stats)
#START_SIZE=`du -s -Bk . | awk '{ print substr($1, 1, length($1)-1) }'`
COUNT=1
NUM_FILES=0
TAPE_ALREADY_DONE=0

while not_end_of_tape
do
    # grabbing stuff
    if [ $NUM_FILES -eq 0 ]; then
        echo -ne "Reading tape filemark $COUNT\r"
    else
        echo -ne "Reading tape filemark $COUNT of $NUM_FILES\r"
    fi

    #if ! mbuffer -q -s 256k -m 128M -i /dev/nst0 | tar -b 512 -xf -; then
    if ! tar -b 512 -xf /dev/nst0; then
        if not_end_of_tape; then
            echo "Error untarring file"
            exit -1
        fi
        break
    fi 
    if ! expr $COUNT % 50 > /dev/null; then
        let SECONDS_SO_FAR=`date +%s`-$START_SECONDS 
        echo "To filemark $COUNT took "$SECONDS_SO_FAR" seconds    "
    fi
    if [ $COUNT -eq 1 ]; then
        TOCNAME=`ls -ltc ./*.toc | awk ' { if (NR==1) print $9 }'`
        TAPENAME=${TOCNAME:(-12):8}

        # if unloading from tape changer the tapename will have
        # been passed in and we can do an extra check to ensure
        # tocname matches tapename
        if [ $# -gt 0 ]; then
            if [ "$1" != "$TAPENAME" ]; then
                echo "ERROR: tapename is $1 but toc file is $TOCNAME"
                echo "(review this may be a label swap)"
                exit -1
            fi 
        fi
        
        # check if tape already been done
        while read LINE
        do
            if [ "$LINE" == "$TAPENAME" ]; then
                echo "Tape $TAPENAME has already been unarchived"
                let TAPE_ALREADY_DONE=1
                break
            fi
        done < ./tape_list.log

        if [ $TAPE_ALREADY_DONE -eq 0 ]; then
            NUM_FILES=`awk -v target="$TAPENAME" '/<FileList/ { filelist=1; } /<Tape>/ { if ((filelist == 1) && (target == substr($1, 7, length($1)-13))) tape=1; else tape=0; } /<TapeFilemark/ { if (tape == 1) mark=substr($2, 5)+0 } END{print mark}' $TOCNAME`
            echo "Tape $TAPENAME has $NUM_FILES tar files"
        else
            break
        fi
    fi

    # skipping file mark
    #mbuffer -q -s 256k -m 128M -i /dev/nst0 | tar -b 512 -xf -
    tar -b 512 -xf /dev/nst0 2> /dev/null
    let COUNT=$COUNT+1
done

# print stats
# REVIEW: currently disabled as du can take ages on a SAN
let TOTAL_SECONDS=`date +%s`-$START_SECONDS
#END_SIZE=`du -s -Bk . | awk '{ print substr($1, 1, length($1)-1) }'`
#let TOTAL_SIZE=$END_SIZE-$START_SIZE
#let SPEED=$TOTAL_SIZE/$TOTAL_SECONDS
if [ $TAPE_ALREADY_DONE -eq 0 ]; then
    echo
    echo "Tape offload took $TOTAL_SECONDS seconds"
#if [ $SPEED -ge 1000 ]; then
#    let SPEED=$SPEED/1000
#    echo "Tape offload speed is $SPEED MB/s"
#else 
#    echo "Tape offload speed is $SPEED KB/s"
#fi
fi

# move new files in root directory into tapename folder
echo "tidying up"
NEW_FILES=`ls`
for n in $NEW_FILES; do
    FOUND=0
    for o in $OLD_FILES; do
        if [ $o == $n ]; then
            FOUND=1
        fi
    done
    if [ $FOUND -eq 0 ]; then
        if [ ! -d $n ]; then
            [ ! -d $TAPENAME ] && mkdir $TAPENAME
            cp $n $TAPENAME/$n
        fi
    fi
done
echo $TAPENAME >> ./tape_list.log

echo "done"

