#!/bin/sh

COUNT=0
ONE_HIT_WONDER=0
ARG_ERROR=0
for i in $@
do
    if [ $i = "-onehit" ]; then
        if [ $COUNT -eq 0 ]; then
            ONE_HIT_WONDER=1
        else
            ARG_ERROR=1
            break
        fi
    else
        AWK_ARGS="$AWK_ARGS $i"
    fi
    let COUNT=$COUNT+1
done

if [ $# -eq 0 -o $ARG_ERROR -eq 1 ]; then
    echo "findtape.sh [-onehit] [search-criteria1] ...."
    echo "-onehit only return newest tape with matching material on"
    echo "Where search criteria can be one or move of:"
    echo "-v rollname=\"<roll>\""
    echo "-v shotname=\"<name>\""
    echo "-v timecode1=\"<start tc, e.g. 01:00:00:00>\""
    echo "-v timecode2=\"<end tc, e.g. 02:00:00:00>\""
    echo "-v fps=\"<frame rate - default 24>\""
    echo "-v ignorecase=\"<1 (on) or 0 (off): default is case sensitive >\""
    echo "-v handle=\"<number of frames in start/end handles - default 0; v2+ TOC only>\""
    echo "-v verbose=\"<1 (on) or 0 (off): default off>\""
    echo "-v shotlevel=\"<1 (on) or 0 (off - default): list entire shot on a match>\""
    echo "-v firstHitOnly=\"<1 (on) or 0 (off - default): only list first shot match>\""
    echo "Search criteria are AND'd together apart from timecodes which"
    echo "specify a start (and optional end) timecode."
    echo "Results will be tape names and filemarks or ranges of filemarks."
    echo "Table of contents files from tapes must be copied to /etc/codex"
    exit -1
fi

# list toc files under var log in order - latest first
echo "Searching for TOCs..."
find /etc/codex/tape-offloads -type f -print0 | 
    xargs -0 ls -l --time-style=long-iso | 
    grep .toc$ | 
    awk '{ print $6" "$7" "$8 }' | 
    sort -nr 2> /dev/null | 
    awk '{ print $3 }' > ./toc.lst

echo "Discarding old TOCs for tapes used again..."
awk ' /.toc/ {
    # strip out tape name
    toc = substr($0, length($0) - 11, 12);

    # only output tape name if not yet in list
    if (!index(list, toc))
    {
        print $0;
   
        # add to list
        temp = list " " toc;
        list = temp;
    }
}' ./toc.lst > ./toc.srt

# go through files in list and call awk search on them
echo "Parsing TOCs..."
COUNT=1
HITS=0
echo "Results:" > ./toc.res
TOTAL_NUM_TOCS=`cat ./toc.lst | wc -l`
NUM_FILES=`cat ./toc.srt | wc -l`

for i in `cat ./toc.srt`;
do
    printf "progress: %d/%d: %-60s\r" $COUNT $NUM_FILES $i 

    if [ -f toc.hit ]; then
        rm -f ./toc.hit
    fi

    # check friendly shotlist in v2+ tocs if present to determine
    # what files are for what shots on the tape
    if grep "TOCVersion" $i > /dev/null; then
	awk -f /home/codex/toc2.awk $AWK_ARGS $i > ./toc.hit
    else
        awk -f /home/codex/toc.awk $AWK_ARGS $i >> ./toc.hit
    fi

    SIZE=`ls -l ./toc.hit | awk '{ print $5 }'`
    if [ $SIZE -gt 1 ]; then
        echo `ls -l $i | awk '{ print "TOC: "$6" "$7" "$8 }'`" ($i)" >> ./toc.res
        cat ./toc.hit >> ./toc.res
        let HITS=$HITS+1
        if [ $ONE_HIT_WONDER -eq 1 ]; then
            break
        fi
    fi

    let COUNT=$COUNT+1

done
echo "Scanning complete...                                                           "

rm -f toc.lst
rm -f toc.hit
rm -f toc.srt

if [ $HITS -eq 0 ]; then
    echo "No matches found." >> ./toc.res
fi

echo "$NUM_FILES files checked ($TOTAL_NUM_TOCS files found)." >> ./toc.res

cat ./toc.res

rm -f toc.res

if [ $HITS -eq 0 ];then
    exit -1
fi

