#/bin/sh

# Parse the command line arguments...

GOOD_ARGS=0
END_OFFSET=0
START_OFFSET=0
DISKSPACE_CHECK=1

if [ $# -ge 2 ]; then

    if [ $# -le 12 ]; then

        TAPE_NAME=$1
        START_FILEMARK=$2
        END_FILEMARK=$2

        GOOD_ARGS=1

        # check for optional arguments
        START_OFFSET_FLAG=0
        END_OFFSET_FLAG=0
        EXPLICIT_END_FILEMARK_FOUND=0
        DUPE=0
        LAST=0
        for i in $@
        do
            let COUNT=$COUNT+1
            if [ $COUNT -gt 2 ]; then
                if [ $i = "-s" ]; then
                    START_OFFSET_FLAG=1
                elif [ $i = "-e" ]; then
                    END_OFFSET_FLAG=1
                elif [ $i = "-d" ]; then
                    DUPE=1
                elif [ $i = "-f" ]; then
                    DISKSPACE_CHECK=0
                elif [ $i = "-l" ]; then
                    LAST=1
                else
                    if [ $START_OFFSET_FLAG -eq 1 ]; then
                        START_OFFSET=$i
                        START_OFFSET_FLAG=0
                    elif [ $END_OFFSET_FLAG -eq 1 ]; then
                        END_OFFSET=$i
                        END_OFFSET_FLAG=0
                    elif [ $DUPE -eq 1 ]; then
                        DUPE=0
                        DUPETAPE=$i
                    elif [ $LAST -eq 1 ]; then
                        LAST=0
                        if [ -f ./lastget.good ]; then
                            LASTTAPE=$i
                        fi
                    elif [ $EXPLICIT_END_FILEMARK_FOUND -eq 0 ]; then
                        END_FILEMARK=$i
                        EXPLICIT_END_FILEMARK_FOUND=1
                    else
                        GOOD_ARGS=0
                    fi
                fi
            fi
        done

    fi
fi
rm -f ./lastget.good

if [ $GOOD_ARGS -eq 0 ]; then
    echo "getframes.sh <tape> <start> [<end>] [-s <offset>] [-e <offset>] [-f]"
    echo "             [-d <tapename>] [-l <tapename>"
    echo "Where:"
    echo " <tape> = tape roll name"
    echo " <start> = filemark to start unarchiving from (if no end then only filemark)"
    echo " <end> = unarchive from start to this filemark" 
    echo " -s <n>= offset into start filemark's files - delete files before this number"
    echo " -e <n>= offset into end filemark's files - delete files after this number"
    echo " -f = force unarchive: don't insist on 5GB free to unarchive to system disk"
    echo " -d = duplicate tape name"
    echo " -l = last tape name"
    exit -1
fi

function checkDiskspace {
    # diskspace check
    if [ $DISKSPACE_CHECK -eq 1 ]; then

	df -BG . | awk '
            { if (NR==2)
                {
                    if ($1 == "/dev/sda1")
                        print "sys disk";
                }     
                
            }' > sysdisk.tmp
        if grep "sys disk" sysdisk.tmp > /dev/null; then
            df -BG . | awk '
                { if (NR==2) 
                    {
                        if (($4 == "0G") || ($4 == "1G")) 
                            print "empty"; 
                    }
                }' > diskspace.tmp
            if grep "empty" diskspace.tmp > /dev/null; then
                echo
                echo "Aborting.  There are less than 5GB left on the system disk which is "
                echo "the current unarchive destination."
                echo "This behaviour can be overwritten using the -f argument."
                echo "BEWARE: filling up the system disk up can result in this machine"
                echo "becoming unstable or unuseable."
                echo
                rm -f diskspace.tmp
                exit -1
            fi
            rm -f diskspace.tmp
        fi
        rm -f sysdisk.tmp
    fi
}

checkDiskspace

function tidyUp {
    [ -f ./mt.tmp ];              rm -f ./mt.tmp
    [ -f ./mtx.tmp ];             rm -f ./mtx.tmp
    [ -f ./tape_stat.tmp ];       rm -f ./tape_stat.tmp
    [ -f ./tape_slot.tmp ];       rm -f ./tape_slot.tmp
    [ -f ./drive0_stat.tmp ];     rm -f ./drive0_stat.tmp
    [ -f ./drive1_stat.tmp ];     rm -f ./drive1_stat.tmp
    [ -f ./drive0_src_slot.tmp ]; rm -f ./drive0_src_slot.tmp
    [ -f ./error.tmp ];           rm -f ./error.tmp
}
# get changer status
TAPE_CHANGER_PRESENT=1
mtx status 1> ./mtx.tmp 2> error.tmp
if grep "cannot open SCSI device" ./error.tmp > /dev/null; then
    TAPE_CHANGER_PRESENT=0
fi

DRIVE_NUMBER=-1
if [ $TAPE_CHANGER_PRESENT -eq 1 ]; then

    # is the tape in the changer?
    # And if not is there a duplicate that is present?
    if ! grep $TAPE_NAME ./mtx.tmp > ./tape_stat.tmp; then
        if [ "$DUPETAPE" != "" ]; then
            if grep $DUPETAPE ./mtx.tmp > ./tape_stat.tmp; then
                let TAPE_NAME=$DUPETAPE
            fi
        fi
    fi
    if grep $TAPE_NAME ./mtx.tmp > ./tape_stat.tmp; then

        # is the tape in one of the drives?
        if grep "Data Transfer Element 0" ./tape_stat.tmp > /dev/null; then
            echo Tape $TAPE_NAME already in nst0
            DRIVE_NUMBER=0
        elif grep "Data Transfer Element 1" ./tape_stat.tmp > /dev/null; then
            echo Tape $TAPE_NAME already in nst1
            DRIVE_NUMBER=1
        else
           # tape not in a drive - find the slot number
           awk '{ print substr($3, 1, index($3,":")-1); }' ./tape_stat.tmp > ./tape_slot.tmp
           STORAGE_SLOT=`cat ./tape_slot.tmp`

           # find an empty drive to load it into
           grep "Data Transfer Element 0" ./mtx.tmp > drive0_stat.tmp
           if grep "Empty" ./drive0_stat.tmp > /dev/null; then
               echo "Loading tape $TAPE_NAME into nst0 from slot $STORAGE_SLOT..."
               mtx load $STORAGE_SLOT 0
               DRIVE_NUMBER=0
           else
               UNLOAD=1
               if grep "Data Transfer Element 1" ./mtx.tmp > drive1_stat.tmp; then
                   if grep "Empty" ./drive1_stat.tmp > /dev/null; then
                       echo "Loading tape $TAPE_NAME into nst1 from slot $STORAGE_SLOT..."
                       mtx load $STORAGE_SLOT 1
                       DRIVE_NUMBER=1
                       UNLOAD=0
                   fi
               fi

               # if no free drive found then unload tape from 
               # drive 0
               if [ $UNLOAD -eq 1 ]; then
                   awk '{ print $7; }' drive0_stat.tmp > ./drive0_src_slot.tmp
                   DRIVE0_SRC_SLOT=`cat ./drive0_src_slot.tmp`
                   echo "Unloading wrong tape from nst0..."
                   mtx unload $DRIVE0_SRC_SLOT 0
                   echo "loading tape $TAPE_NAME into nst0 from slot $STORAGE_SLOT..."
                   mtx load $STORAGE_SLOT 0
                   DRIVE_NUMBER=0
               fi
           fi
        fi
    else
        echo "$TAPE_NAME is not in the tape changer."
        echo "Please insert then retry script."
        tidyUp
        exit -1   
    fi
else
    if [ "$DUPETAPE" != "" ]; then
        TEMP=$TAPE_NAME" or "$DUPETAPE
    else
        TEMP=$TAPE_NAME
    fi
    mt -f /dev/nst0 status > ./mt.tmp 2> /dev/null
    if [ ! -s ./mt.tmp ]; then
        echo "Unable to obtain tape drive status - is the drive connected?"
        tidyUp
        exit -1
    elif grep "DR_OPEN" ./mt.tmp > /dev/null; then
        echo "No tape in drive - please insert tape $TEMP into nst0 and retry."
        tidyUp
        exit -1
    else
        DOCHECK=1
        if [ "$LASTTAPE" != "" ]; then
            if [ "$TAPE_NAME" = "$LASTTAPE" ] || \
                               [ "DUPETAPE" = "LASTTAPE" ]; then
                let DOCHECK=0
            fi
        fi
        if [ $DOCHECK -eq 1 ]; then
            echo -n "Is tape $TEMP in drive nst0 ([Y]es/[N]o/[S]kip) ? "
            read -e ANSWER
            if [ "$ANSWER" == "s" ] || [ "$ANSWER" == "S" ]; then 
                echo "Skipping event"
                exit -1
            elif [ "$ANSWER" != "y" ] && [ "$ANSWER" != "Y" ]; then 
                echo "Please insert tape $TEMP into nst0 then retry script."
                tidyUp
                exit -2
            fi
        fi
    fi
    DRIVE_NUMBER=0
fi

# set defaults
DEVICE_NAME=/dev/nst0
if [ $DRIVE_NUMBER -eq 1 ]; then
    DEVICE_NAME=/dev/nst1
fi

# As we have variable block sizes we need check if we know the 
# blocksize of this tape already from the file storing tape
# blocksizes - if not we'll rewind it and attempt 64k and 256k
# blocksizes
# n.b. this doesn't use TOC file parsing (which could have block
# sizes in) because this can be done irrespective of TOCs so why
# use them?
TAPE_BLOCK_SIZE_FILE="/etc/codex/tape-offloads/getframes.tapelist"
UNKNOWN_TAPE=1
if [ -f $TAPE_BLOCK_SIZE_FILE ]; then 
    BLOCK_SIZE_ARG=`grep $TAPE_NAME $TAPE_BLOCK_SIZE_FILE | awk '{ print $2; }'`
    if [ $BLOCK_SIZE_ARG ]; then
        UNKNOWN_TAPE=0
    fi
fi

mt -f $DEVICE_NAME status > ./mt.tmp 2> /dev/null
MARK=`awk '/File number=/ { print substr($2, 8, length($2)-8)+0; }' mt.tmp`
BLOCK=`awk '/File number=/ { print substr($4, 8, length($4)-8)+0; }' mt.tmp`
if [ $MARK -eq -1 -o $UNKNOWN_TAPE -eq 1 ]; then
    echo "Rewinding tape in "$DEVICE_NAME"..."
    mt -f $DEVICE_NAME rewind

    # if unknown tape check block size
    if [ $UNKNOWN_TAPE -eq 1 ]; then
        BLOCK_SIZE_ARG=0
        if tar -b 128 -tf $DEVICE_NAME > /dev/null 2>&1; then
            BLOCK_SIZE_ARG=128
        else
            mt -f $DEVICE_NAME rewind
            if tar -b 512 -tf $DEVICE_NAME > /dev/null 2>&1; then
                BLOCK_SIZE_ARG=512
            fi
        fi
        if [ $BLOCK_SIZE_ARG -ne 0 ]; then
            echo "$TAPE_NAME $BLOCK_SIZE_ARG" >> $TAPE_BLOCK_SIZE_FILE
        fi
    fi

    # fwd to the desired tape mark
    if [ $START_FILEMARK -gt 0 ]; then
        echo "Fast forwarding to tape mark $START_FILEMARK..."
        mt -f $DEVICE_NAME fsf $START_FILEMARK
    fi
else
    if [ $START_FILEMARK -gt $MARK ]; then
        let DIFF=$START_FILEMARK-$MARK
        echo "Fast forwarding $DIFF marks to tape mark $START_FILEMARK..."
        mt -f $DEVICE_NAME fsf $DIFF
    elif [ $MARK -gt $START_FILEMARK ]; then
        let DIFF=$MARK-$START_FILEMARK
        let DIFF=$DIFF+1
        echo "Rewinding $DIFF marks to tape mark $START_FILEMARK..."
        mt -f $DEVICE_NAME bsfm $DIFF
    elif [ $BLOCK -ne 0 ]; then
        echo "Not at start of tape mark $START_FILEMARK - repositioning..."
        mt -f $DEVICE_NAME bsfm 1
    fi
fi

# do the untar
echo "Unarchiving files..."
mt -f $DEVICE_NAME setblk 0
let NUMBER_ITERATIONS=$[$END_FILEMARK-$START_FILEMARK+1]
for ((i=$START_FILEMARK; i<=$END_FILEMARK; i++)); do

    checkDiskspace

    NUM_FILES_DONE=$[$i-$START_FILEMARK]
    PROGRESS=$[($NUM_FILES_DONE*100) / $NUMBER_ITERATIONS]
    echo -en "Completed: $PROGRESS%  \r"

    # do we need to delete any files from this chunk?
    SOMETHING_TO_DELETE=0
    if [ $i -eq $END_FILEMARK -a $END_OFFSET -gt 0 ]; then
        SOMETHING_TO_DELETE=1
    elif [ $i -eq $START_FILEMARK -a $START_OFFSET -gt 0 ]; then
        SOMETHING_TO_DELETE=1
    fi

    # if deleting files from this chunk then create a file
    # with delete commands in
    # N.B. --no-same-owner prevents complaints offloading to FAT32
    if [ $SOMETHING_TO_DELETE ]; then
        tar -b $BLOCK_SIZE_ARG --no-same-owner -xvf $DEVICE_NAME | awk '{ print "rm -f " $1; }' > del.tmp
    else
        tar -b $BLOCK_SIZE_ARG --no-same-owner -xvf $DEVICE_NAME 
    fi
 
    # if something to delete then create scripts with commands
    # to delete the appropriate files
    if [ $SOMETHING_TO_DELETE -gt 0 ]; then

        if [ $i -eq $START_FILEMARK -a $START_OFFSET -gt 0 ]; then
            cat del.tmp | awk -v cutoff=$START_OFFSET '{ if (NR<=cutoff) print $0; }' > startdel.sh
            chmod 755 startdel.sh
            ./startdel.sh
            rm -f startdel.sh
        fi

        if [ $i -eq $END_FILEMARK -a $END_OFFSET -gt 0 ]; then
            NUM_FILES_IN_CHUNK=`cat del.tmp | wc -l`
            CUTOFF=$[$NUM_FILES_IN_CHUNK-$END_OFFSET]
            cat del.tmp | awk -v cutoff=$CUTOFF '{ if (NR>cutoff) print $0; }' > enddel.sh
            chmod 755 enddel.sh
            ./enddel.sh
            rm -f enddel.sh
        fi

        rm -f del.tmp
    fi

    # need to read in the filemark between files from tape to 
    # move on to next real file (don't need to do for
    # last one as we've finished)
    if [ $i -ne $END_FILEMARK ]; then
        tar -b $BLOCK_SIZE_ARG -xf $DEVICE_NAME
    fi

done
echo "Frame unarchiving completed.           "

tidyUp
echo 1 > ./lastget.good
