/*
**  @(#) 1.6 src/ltfs_error.h, LTFS, ltfs.01.02.00, 201102171 2/15/11 02:22:54
**
**  ZZ_Copyright_BEGIN
**  
**  
**  Licensed Materials - Property of IBM
**  
**  IBM Linear Tape File System Single Drive Edition
**  
**  Copyright IBM Corp. 2010, 2011
**  
**  This file is part of the IBM Linear Tape File System Single Drive Edition
**  (formally known as IBM Long Term File System)
**  
**  The IBM Linear Tape File System Single Drive Edition is free software;
**  you can redistribute it and/or modify it under the terms of the GNU Lesser 
**  General Public License as published by the Free Software Foundation, 
**  version 2.1 of the License.
**  
**  The IBM Linear Tape File system Single Drive Edition is distributed in the 
**  hope that it will be useful, but WITHOUT ANY WARRANTY; without even the 
**  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**  See the GNU Lesser General Public License for more details.
**  
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
**  or download the license from <http://www.gnu.org/licenses/>.
**  
**  
**  ZZ_Copyright_END
**
*************************************************************************************
**
** COMPONENT NAME:  IBM Linear Tape File System
**
** FILE NAME:       ltfs_error.h
**
** DESCRIPTION:     Error codes for libltfs.
**
** AUTHORS:         Brian Biskeborn
**                  IBM Almaden Research Center
**                  bbiskebo@us.ibm.com
**
*************************************************************************************
*/

#ifndef __ltfs_error_h__
#define __ltfs_error_h__

#define LTFS_ERR_MIN          1000  /* First error value defined by this file */

/* Internal errors */
#define LTFS_NULL_ARG         1000  /* Unexpected NULL function argument */
#define LTFS_NO_MEMORY        1001  /* Memory allocation failed */
#define LTFS_MUTEX_INVALID    1002  /* Attempted to lock or unlock an uninitialized mutex */
#define LTFS_MUTEX_UNLOCKED   1003  /* Attempted to unlock an already unlocked mutex */
#define LTFS_BAD_DEVICE_DATA  1004  /* Invalid device data structure */
#define LTFS_BAD_PARTNUM      1005  /* Invalid partition number requested */
#define LTFS_LIBXML2_FAILURE  1006  /* A libxml2 call failed */

/* Device errors */
#define LTFS_DEVICE_UNREADY   1007  /* Device is not ready */
#define LTFS_NO_MEDIUM        1008  /* No medium present in the device */
#define LTFS_LARGE_BLOCKSIZE  1009  /* Device does not support the formatted blocksize */
#define LTFS_BAD_LOCATE       1010  /* Locate resulted in an unexpected position */

/* LTFS format errors */
#define LTFS_NOT_PARTITIONED  1011  /* Medium contains only one partition */
#define LTFS_LABEL_INVALID    1012  /* Invalid partition label */
#define LTFS_LABEL_MISMATCH   1013  /* Partition labels do not match */
#define LTFS_INDEX_INVALID    1014  /* Invalid XML index or index backpointer chain. Recovery might be possible. */
#define LTFS_INCONSISTENT     1015  /* Volume is inconsistent but recoverable */

#define LTFS_UNSUPPORTED_MEDIUM 1016  /* Medium cannot be partitioned or is otherwise incompatible */
/* 1017 - 1019 is already used in version2*/

#define LTFS_POLICY_EMPTY_RULE 1020 /* Empty name rule encountered during policy parsing */
#define LTFS_MUTEX_INIT         1021  /* Mutex initialization failed */
#define LTFS_BAD_ARG            1022  /* Generic error for an invalid function argument */
/* 1023 - 1033 is already used in version2*/

#define LTFS_EOD_MISSING_MEDIUM       1034  /* Medium has no EOD in one of partitions*/
#define LTFS_BOTH_EOD_MISSING         1035  /* Medium has no EOD in both partitions*/
#define LTFS_UNEXPECTED_VALUE         1036  /* Detect Unexpected Value in LTFS in itself */
#define LTFS_UNSUPPORTED              1037  /* Call unsupported method */
#define LTFS_READ_ERROR               1038  /* Unrecoverable read error */
#define LTFS_DETECT_EOD               1039  /* Unrecoverable read error */
/* 1040 - 1042 is already used in version2*/
#define LTFS_UNSUPPORTED_INDEX_VERSION 1043  /* Unsupported version is detected into index */

/* 2xxxx are reserved for device error codes */


#endif /* __ltfs_error_h__ */
