/*
**  @(#) 1.4 src/ltfs_types.h, LTFS, ltfs.01.02.00, 201102171 2/15/11 02:23:03
**
**  ZZ_Copyright_BEGIN
**  
**  
**  Licensed Materials - Property of IBM
**  
**  IBM Linear Tape File System Single Drive Edition
**  
**  Copyright IBM Corp. 2010, 2011
**  
**  This file is part of the IBM Linear Tape File System Single Drive Edition
**  (formally known as IBM Long Term File System)
**  
**  The IBM Linear Tape File System Single Drive Edition is free software;
**  you can redistribute it and/or modify it under the terms of the GNU Lesser 
**  General Public License as published by the Free Software Foundation, 
**  version 2.1 of the License.
**  
**  The IBM Linear Tape File system Single Drive Edition is distributed in the 
**  hope that it will be useful, but WITHOUT ANY WARRANTY; without even the 
**  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**  See the GNU Lesser General Public License for more details.
**  
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
**  or download the license from <http://www.gnu.org/licenses/>.
**  
**  
**  ZZ_Copyright_END
**
*************************************************************************************
**
** COMPONENT NAME:  IBM Linear Tape File System
**
** FILE NAME:       ltfs_types.h
**
** DESCRIPTION:     Type declarations used by several modules.
**
** AUTHORS:         Brian Biskeborn
**                  IBM Almaden Research Center
**                  bbiskebo@us.ibm.com
**
*************************************************************************************
*/

#ifndef __ltfs_types_h__
#define __ltfs_types_h__

typedef uint32_t tape_partition_t;
typedef uint64_t tape_block_t;

struct tc_coherency {
	uint64_t      volume_change_ref; /**< VWJ from the drive */
	uint64_t      count;             /**< Generation of Index */
	uint64_t      set_id;            /**< Position of Index   */
	char          uuid[37];          /**< Volume UUID */
	unsigned char version;           /**< Version field */
};

/* Structure of cartridge health */
#define UNSUPPORTED_CARTRIDGE_HEALTH ((int64_t)(-1))

typedef struct tc_cartridge_health {
	int64_t  mounts;           /* Total number of mount in the volume lifetime */
	uint64_t written_ds;       /* Total number of data sets written in the volume lifetime */
	int64_t  write_temps;      /* Total number of recoverd write error in the volume lifetime */
	int64_t  write_perms;      /* Total number of unrecoverd write error in the volume lifetime */
	uint64_t read_ds;          /* Total number of data sets read in the volume lifetime */
	int64_t  read_temps;       /* Total number of recoverd read error in the volume lifetime */
	int64_t  read_perms;       /* Total number of unrecoverd read error in the volume lifetime */
	int64_t  write_perms_prev; /* Unrecoverd write errors in previous mount */
	int64_t  read_perms_prev;  /* Unrecoverd read errors in previous mount */
	uint64_t written_mbytes;   /* Total number of mega bytes written in the volume lifetime */
	uint64_t read_mbytes;      /* Total number of mega bytes read in the volume lifetime */
	int64_t  passes_begin;     /* Count of the total number of times the beginning of medium position has passed */
	int64_t  passes_middle;    /* Count of the total number of times the middle of medium position has passed */
	uint64_t tape_alert;       /* Tape alert showed by 64 bits flag, MSB is tape alert 64, LSB is tape alert 1*/
	int64_t  tape_efficiency;  /* Tape efficiency (0-255) */
} cartridge_health_info;

#endif /* __ltfs_types_h__ */
