/*
**  @(#) 1.2 src/plugin.h, LTFS, ltfs.01.02.00, 201102171 2/15/11 02:23:23
**
**  ZZ_Copyright_BEGIN
**  
**  
**  Licensed Materials - Property of IBM
**  
**  IBM Linear Tape File System Single Drive Edition
**  
**  Copyright IBM Corp. 2010, 2011
**  
**  This file is part of the IBM Linear Tape File System Single Drive Edition
**  (formally known as IBM Long Term File System)
**  
**  The IBM Linear Tape File System Single Drive Edition is free software;
**  you can redistribute it and/or modify it under the terms of the GNU Lesser 
**  General Public License as published by the Free Software Foundation, 
**  version 2.1 of the License.
**  
**  The IBM Linear Tape File system Single Drive Edition is distributed in the 
**  hope that it will be useful, but WITHOUT ANY WARRANTY; without even the 
**  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**  See the GNU Lesser General Public License for more details.
**  
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
**  or download the license from <http://www.gnu.org/licenses/>.
**  
**  
**  ZZ_Copyright_END
**
*************************************************************************************
**
** COMPONENT NAME:  IBM Linear Tape File System
**
** FILE NAME:       plugin.h
**
** DESCRIPTION:     Declares functions for working with libltfs plugins.
**
** AUTHORS:         Brian Biskeborn
**                  IBM Almaden Research Center
**                  bbiskebo@us.ibm.com
**
*************************************************************************************
*/

#ifndef __PLUGIN_H__
#define __PLUGIN_H__

#include "config_file.h"

struct libltfs_plugin {
	void *lib_handle;  /**< Handle to the shared library implementing the plugin */
	void *ops;         /**< Pointer to the plugin's function list */
	void *messages;    /**< Handle to the plugin message bundle generated by ltfsprintf.c */
};

/**
 * Load a plugin (a shared library that implements a particular set of operations).
 * This function may be called multiple times for the same plugin.
 * It is not reentrant.
 * @param pl Plugin structure to populate.
 * @param type Plugin type, must be "iosched" or "driver".
 * @param name The name of the plugin to load.
 * @param config Configuration structure to search for the plugin path.
 * @return 0 on success or a negative value on error.
 */
int plugin_load(struct libltfs_plugin *pl, const char *type, const char *name,
	struct config_file *config);

/**
 * Unload a previously loaded plugin. This should be called once for each call to @plugin_load.
 * This function is not reentrant.
 * @param pl Plugin to unload.
 * @return 0 on success or a negative value on error.
 */
int plugin_unload(struct libltfs_plugin *pl);

#endif /* __PLUGIN_H__ */

