#/bin/sh

if [ $# -lt 3 ]; then
    echo "ARCHIVE FAILED: wrong number of arguments"
    exit -1
fi

# Set Internal Field Separator to NOT include space so that any
# filenames with spaces in aren't taken to be multiple files when
# tokenising
IFS=$'\n'

# Strip out variables from arguments passed in to the script
# TAPE_DEV = 1st argument
# MD5_FILENAME = 2nd argument
# TAR_ARGUMENTS = 3rd argument onwards (as an array)
index=0
declare -a TAR_ARGUMENTS
for arg in "$@"
do
    if [ "$index" -eq 0 ]; then
        TAPE_DEV=$arg
    elif [ "$index" -eq 1 ]; then
        MD5_FILENAME=$arg
    else
        TAR_ARGUMENTS[($index-1)]=$arg
    fi

    index=$(( $index + 1))
done

# set hardware compression on - can give slight speed up depending on material
mt -f $TAPE_DEV compression 1

# set to variable block size at hardware level - this seems to give best
# behaviour on different machines with LTO3 and LTO4 media.
# N.B. tape machines are a law unto themselves and so this is a bit
# 'suck it and see'
mt -f $TAPE_DEV setblk 0

# Do the business!
# Create a tar, using a buffer to smooth data rate to tape to help
# prevent shoe-shining (tape drive moving backwards and forwards
# when data rate changes).
# -b 128 means 128x512 bytes is the tar block size (i.e. 64k)
# -s 64k means mbuffer is using 64k blocks to match
# mbuffer also outputs a md5 checksum with the -H option which
# is compared to during verification.
# N.B. using the commented out line below with the R option would
# produce a block number in the output - this would be used to
# produce a block log file as requested by Jon F at Sohonet to use
# for shuttling more efficiently when unarchiving but we have not
# tested this yet.
# CR! 01/10/09 
#tar -b 128 -cRvf - ${TAR_ARGUMENTS[@]} | \

# 64k blocks
#tar -b 128 -cvf - ${TAR_ARGUMENTS[@]} | \
#    mbuffer -q -s 64k -m 128M -H $MD5_FILENAME.md5sum -o $TAPE_DEV

# 256k blocks
tar -b 512 -cvf - ${TAR_ARGUMENTS[@]} | \
    mbuffer -q -s 256k -m 128M -H $MD5_FILENAME.md5sum -o $TAPE_DEV

