#!/bin/sh

if [ $# != 2 ]; then
    echo "VERIFY FAILED: not correct number of arguments"
    exit
fi

TAPE=$1
FILE=$2

if [ ! -s $FILE.md5sum ]; then
    echo "VERIFY $FILE: FAILED no source checksum found"
    exit
fi

echo "Recreating checksum for $FILE from tape..."

# set compression on although don't think this does owt when reading
mt -f $TAPE compression 0

# set hardware block size to variable (0) which seems to give best results
mt -f $TAPE setblk 0

# read from tape in 64k block size (size laid down onto tape via tapearchive
# on a variable block size tape)
#mbuffer -q -s 64k -m 128M -i $TAPE -o /dev/null -H $FILE.md5check

# 256k blocks
mbuffer -q -s 256k -m 128M -i $TAPE -o /dev/null -H $FILE.md5check

if [ ! -s $FILE.md5check ]; then
    echo "VERIFY $FILE: FAILED no validation checksum created"
    exit
fi

cmp $FILE.md5sum $FILE.md5check > $FILE.res

if [ -s $FILE.res ]; then 
    echo "VERIFY $FILE: FAILED checksum mismatch"
else
    echo "VERIFY $FILE: OK"
fi

rm $FILE.res
