# toc.awk
# for scanning codex tape offload table of contents files and finding
# which tape and at which tape marks a shot/roll/timecode's dpx frames
# are
# assuming format is <rollname>/dpx/<rollname>_<name>_<uuid>/<filename>
# (will also fine other file types if stored in same format where the
# extension is same length as 'dpx' - e.g. xml, wav)

# calculate frames since midnight from timecode
NR == 1 {
    mask = 0;
    info_string = "";
    master_string = "";
    cur_tape = "";
    roll_hit = 1;
    anytc_hit = 2;
    name_hit = 4;
    hitmask = 0;

    if (shotname != "")
        hitmask += name_hit;
    
    if (rollname != "")
        hitmask += roll_hit;

    if (ignorecase == "")
        ignorecase = 0;

    if (fps == 0)
        fps = 24;

    if (timecode1 != 0)
    {
        frames_since_midnight1 = convert_timecode_to_frames(timecode1);
        hitmask += anytc_hit;
    }

    if (timecode2 != 0)
    {
        frames_since_midnight2 = convert_timecode_to_frames(timecode2);
        if (timecode1 == 0)
            hitmask += anytc_hit;
    }
    else
        frames_since_midnight2 = -1;

    frames_per_day = fps * 24 * 60 * 60;
}

function check_frames(event_start_frames, start_timecode, 
                      event_end_frames, end_timecode)
{
    if (event_start_frames != 0)
    {
        filemark_start_frames = get_frames_from_line();
        if (filemark_start_frames == event_start_frames)
        {
            mask += anytc_hit;
            info_string = "timecode match " start_timecode 
                          " (" event_start_frames ")";
            hit(info_string);
        }
        else if (filemark_start_frames == event_end_frames)
        {
            mask += anytc_hit;
            info_string = "timecode match " end_timecode
                          " (" event_end_frames ")"
        }
	else
        {
            # the second revision of v1 files has last filename 
            # written on same line as first in a chunk so try 
            # and extract that
            filemark_end_frames = get_end_frames_from_line();
            if (filemark_end_frames)
            {
                if (event_start_frames+0 <= filemark_end_frames+0)
                {
                    if ( ((event_end_frames+0 == -1) && 
                          (event_start_frames+0 >= filemark_start_frames+0)) ||
                         (event_end_frames+0 >= filemark_start_frames+0) )
                    {
                        mask += anytc_hit;
                        info_string = "timecode match " start_timecode 
                                      " (" event_start_frames ")";
                        hit(info_string);
                    }
                }
            }
        }
    }
}

function convert_timecode_to_frames(timecode)
{
    split(timecode, timecode_comps, ":");
    frames = strtonum(timecode_comps[1]);
    frames *= 60;
    frames += strtonum(timecode_comps[2]);
    frames *= 60;
    frames += strtonum(timecode_comps[3]);
    frames *= fps;
    frames += strtonum(timecode_comps[4]);
    return frames;
}

function strip_whitespaces(in_string)
{
    out_string = in_string
    sub(/^[ \t\n\r]+/, "", out_string);
    sub(/[ \t\n\r]+$/, "", out_string);
    return out_string;
}

function parseFilenameFromLine()
{
    line = strip_whitespaces($0);
    return line;
}

function get_rollname_from_line()
{
    # take roll to be first directory name
    line = parseFilenameFromLine();
    end_roll_index = index(line, "/");
    # handle if filename starts with a / or not
    if (end_roll_index == 1)
    {
        line = substr(line, 2, length(line) - 1);
        end_roll_index = index(line, "/");
    }
    our_roll = substr(line, 1, end_roll_index-1);
    return our_roll;
}

function get_shotname_from_line()
{
    our_shotname = "";
    our_rollname = get_rollname_from_line();
    if (length(our_rollname))
    {
        # assuming format is <rollname>/dpx/<rollname>_<name>_<uuid>/<filename>

        # allow for rollname (twice)
        roll_suffix_length = length(our_rollname) * 2;
        # and /dpx/ and the underscore at the end
        roll_suffix_length += 6;
        our_shotname = parseFilenameFromLine();
        if (substr(our_shotname, 1, 1) == "/")
            roll_suffix_length++;
        our_shotname = substr(our_shotname, roll_suffix_length + 1);

        # find end of directory name with shotname in
        finished_index = index(our_shotname, "/");
        if (finished_index == 0)
            finished_index = index(our_shotname, ".");

        # remove "_uuid/" from the end (-38)
        our_shotname = substr(our_shotname, 1, finished_index - 38);
    }
    return our_shotname;
}

function get_frames_from_line()
{
    return get_frames_from_string($0);
}

function get_frames_from_string(source_str)
{
    our_frames_str = "";
    source_str = strip_whitespaces(source_str);
    
    end_index = index(source_str, ".dpx");
    if (end_index)
    {
        start = 0;
        i = end_index - 1;
        while (i > 0)
        {
            if ( (substr(source_str, i, 1) == "_") ||
                 (substr(source_str, i, 1) == ".") )
            {
                start = i + 1;
                break;
            }
            i--;
        }
        if (start > 0)
            our_frames_str = substr(source_str, start, end_index - start);
    }

    ret = our_frames_str+0;
   
    if (ret > frames_per_day)
        ret -= frames_per_day;

    return ret;
}

function get_end_frames_from_line()
{
    our_frames_str = strip_whitespaces($0);

    # first see if there is a first file name on the line
    end_index = index(our_frames_str, ".dpx");
    if (end_index && (end_index < length(our_frames_str) - 4))
    {
        # now look for a end filename
        temp_str = substr(our_frames_str, end_index + 4);
        our_frames_str = get_frames_from_string(temp_str);
    }
    else
        our_frames_str = "";

    ret = our_frames_str+0;
   
    if (ret > frames_per_day)
        ret -= frames_per_day;

    return ret;
}

# results printing function
function hit(info_string)
{
    if (length(master_string) && index(master_string, info_string) == 0)
        master_string = master_string " & " info_string;
    else
        master_string = info_string;
    if (mask != hitmask)
        return;

    # don't print if already shown for this filemark
    if (cur_tape == this_tape && 
            filemark_num != last_filemark_num)
    {
        #if (hit_count == 0)
        #    printf("\ntape %s %s\n", this_tape, FILENAME);
        hit_count++;

        #printf("  filemark %d:\n", filemark_num);
        line = strip_whitespaces($0);
        #printf("    %s\n", line);
        #printf("    %s\n", master_string);
        last_tape_name = tape_name;
        last_filemark_num = filemark_num; 

        tapes[hit_count] = this_tape;
        filemarks[hit_count] = filemark_num;
        lines[hit_count] = $0;
        desc[hit_count] = master_string;
    }
}

mask=0;

# keep track of which tape this toc is for
/This Tape:/ {
    this_tape = $3;
    cur_tape = "";
}

# keep track of current tape details we are examining in the toc
/Slot #/ { 
    cur_tape = $5; 

    # get rid of trailing newline
    sub(/[\n\r]/, "", cur_tape) 
}

# keep track of the current filemark we're at
/Filemark / { 
    filemark_num = $2 
}

# have we found the search term in the current line
# and is it for the tape this toc belongs to and for
# a filemark not yet reported?

tolower($0) ~ tolower(shotname) {
    if ((shotname != "") && (cur_tape == this_tape))
    {
        our_shotname = get_shotname_from_line();

        # handle awk fudging between expressions and constant regexps
        # e.g. shotname of 106+109_111_1 needs to be entered as
        # 106\\\\+109_111_1 for the shotname matching above to work
        # but a \\ has to be removed from when we compare it to 
        # our_shotname - annoying and confusing!
        temp_shotname = shotname;
        sub(/\\/, "", temp_shotname);

        if ( (ignorecase && (tolower(our_shotname)==tolower(temp_shotname)))                 || (our_shotname == temp_shotname) )
        {
            mask += name_hit;
            info_string = "shotname match (" our_shotname ")";
            hit(info_string);
        }
    }
}

tolower($0) ~ tolower(rollname) {
    if ((rollname != "") && (cur_tape == this_tape))
    {
        our_rollname = get_rollname_from_line();
        if ( (ignorecase && (tolower(our_rollname) == tolower(rollname))) ||                      (our_rollname == rollname) )
        {
            mask += roll_hit;
            info_string = "rollname match (" our_roll ")";
            hit(info_string);
        }
    }
}

$0 {
    if (cur_tape == this_tape)
    {
        check_frames(frames_since_midnight1, timecode1, 
                     frames_since_midnight2, timecode2);
    }
}

END {
    i = 1;
    last_filemark_hit = -1;
    first_filemark_hit = -1;
    while (i <= hit_count)
    {
        if (first_filemark_hit == -1)
        {
            printf("Tape: %s marks: ", tapes[i]);
            first_filemark_hit = filemarks[i];
        }
        else if ((filemarks[i] - last_filemark_hit) != 1)
        {
            if (first_filemark_hit == last_filemark_hit)
                printf("%d ", last_filemark_hit);
            else
                printf("%d-%d ", first_filemark_hit, last_filemark_hit);
            first_filemark_hit = filemarks[i];
        }

        last_filemark_hit = filemarks[i];
        i++;
    }
    if (last_filemark_hit == filemarks[hit_count])
    {
        if (first_filemark_hit == last_filemark_hit)
            printf("%d", first_filemark_hit);
        else
            printf("%d-%d", first_filemark_hit, last_filemark_hit);
    }
    if (hit_count > 0)
    {
        if (context)
           printf(" (%s)", context);
        printf("\n");
    }
#printf("v1\n");
}

