# toc2.awk
# for scanning codex tape offload table of contents files and finding
# which tape and at which tape marks a shot/roll/timecode's dpx frames
# are
# This works in version 2.0+ of toc files - i.e. the xml style.
# N.B. works for ARI files too

# calculate frames since midnight from timecode
NR == 1 {
    nextLineTape = false;
    inShot = 0;
    mask = 0;
    info_string = "";
    this_tape = "";
    thisChunkSize = 0;
    thisEndChunkSize = 0;
    filelist_flag = 0;

    roll_hit = 1;
    anytc_hit = 2;
    name_hit = 4;
    hitmask = 0;
    hit_count = 0;

    if (ignorecase == "")
        ignorecase = 0;

    if (shotname != "")
        hitmask += name_hit;
    
    if (rollname != "")
        hitmask += roll_hit;

    if (verbose != "")
        verbose = 1;
    else
        verbose = 0;

    if (shotlevel != "")
        shotlevel = 1;
    else
        shotlevel = 0;

    if (firstHitOnly != "")
        firstHitOnly = 1;
    else
        firstHitOnly = 0;

    if (fps == 0)
        fps = 24;

    if (timecode1 != 0)
    {
        frames_since_midnight1 = convert_timecode_to_frames(timecode1) + 0;
        hitmask += anytc_hit;
    }

    if (timecode2 != 0)
    {
        frames_since_midnight2 = convert_timecode_to_frames(timecode2) + 0;
        if (timecode1 == 0)
            hitmask += anytc_hit;
    }
    else
        frames_since_midnight2 = -1;

    frames_per_day = fps * 24 * 60 * 60;
}

#######################################################################
# Filelist section parsing stuff

function strip_whitespaces(in_string)
{
    out_string = in_string
    sub(/^[ \t\n\r]+/, "", out_string);
    sub(/[ \t\n\r]+$/, "", out_string);
    return out_string;
}

function check_frames(event_start_frames, start_timecode, 
                      event_end_frames, end_timecode)
{
    if (event_start_frames != 0)
    {
        filemark_start_frames = get_frames_from_line();
        if (filemark_start_frames == event_start_frames)
        {
            mask += anytc_hit;
            info_string = "timecode match " start_timecode 
                          " (" event_start_frames ")";
            hit(info_string);
        }
        else if (filemark_start_frames == event_end_frames)
        {
            mask += anytc_hit;
            info_string = "timecode match " end_timecode
                          " (" event_end_frames ")"
        }
        else
        {
            # the second revision of v1 files has last filename 
            # written on same line as first in a chunk so try 
            # and extract that
            filemark_end_frames = get_end_frames_from_line();
            if (filemark_end_frames)
            {
                if (event_start_frames <= filemark_end_frames)
                {
                    if ( ((event_end_frames == -1) && \
                          (event_start_frames >= filemark_start_frames)) || \
                          (event_end_frames >= filemark_start_frames) )
                    {
                        mask += anytc_hit;
                        info_string = "timecode match " start_timecode 
                                      " (" event_start_frames ")";
                        hit(info_string);
                    }
                }
            }
        }
    }
}

function get_rollname_from_filename(filename)
{
    # take roll to be first directory name
    end_roll_index = index(filename, "/");
    # handle if filename starts with a / or not
    if (end_roll_index == 1)
    {
        filename = substr(filename, 2, length(filename) - 1);
        end_roll_index = index(filename, "/");
    }
    our_roll = substr(filename, 1, end_roll_index-1);
    return our_roll;
}

function get_shotname_from_filename(filename)
{
    our_shotname = "";
    our_rollname = get_rollname_from_filename(filename);
    if (length(our_rollname))
    {
        # assuming format is <rollname>/dpx/<rollname>_<name>_<uuid>/<filename>
        # or                 <rollname>/ari/<rollname>_<name>_<uuid>/<filename>

        # allow for rollname (twice)
        roll_suffix_length = length(our_rollname) * 2;
        # and /dpx/ and the underscore at the end
        roll_suffix_length += 6;
        our_shotname = filename;
        if (substr(our_shotname, 1, 1) == "/")
            roll_suffix_length++;
        our_shotname = substr(our_shotname, roll_suffix_length + 1);

        # find end of directory name with shotname in
        finished_index = index(our_shotname, "/");
        if (finished_index == 0)
            finished_index = index(our_shotname, ".");

        # remove "_uuid/" from the end (-38)
        our_shotname = substr(our_shotname, 1, finished_index - 38);
    }
    return our_shotname;
}

function get_frames_from_line()
{
    return get_frames_from_string($0);
}

function get_frames_from_string(source_str)
{
    our_frames_str = "";
    source_str = strip_whitespaces(source_str);
    
    end_index = index(source_str, ".dpx");
    if (!end_index)
	    end_index = index(source_str, ".ari");
    if (end_index)
    {
        start = 0;
        i = end_index - 1;
        while (i > 0)
        {
            if ( (substr(source_str, i, 1) == "_") ||
                 (substr(source_str, i, 1) == ".") )
            {
                start = i + 1;
                break;
            }
            i--;
        }
        if (start > 0)
            our_frames_str = substr(source_str, start, end_index - start);
    }

    ret = our_frames_str+0;

    if (ret > frames_per_day)
        ret -= frames_per_day;

    return ret;
}

function get_end_frames_from_line()
{
    our_frames_str = strip_whitespaces($0);

    # first see if there is a first file name on the line
    end_index = index(our_frames_str, ".dpx");
    if (!end_index)
	    end_index = index(our_frames_str, ".ari");
    if (end_index && (end_index < length(our_frames_str) - 4))
    {
        # now look for a end filename
        temp_str = substr(our_frames_str, end_index + 4);
        our_frames_str = get_frames_from_string(temp_str);
    }
    else
        our_frames_str = "";

    ret = our_frames_str+0;

    if (ret > frames_per_day)
        ret -= frames_per_day;

    return ret;
}

# results printing function
function hit(info_string)
{
    if (mask != hitmask)
        return;
    # don't add if already in list because was in shot list section
    for (i=1; i<=hit_count; i++)
    {
        if (tapes[i] == this_tape)
        {
            # ensure tape_filemark_num is compared as a numeric value with +0
            if ( (tape_filemark_num+0 >= startFilemarks[i]) && \
                            (tape_filemark_num+0 <= endFilemarks[i]) )
            {
                  return;
            }
        }
    }
 
    # don't print if already shown for this filemark
    if (curFileTape == this_tape && \
            tape_filemark_num != last_filemark_num)
    {
        hit_count++;

        last_tape_name = tape_name;
        last_filemark_num = filemark_num; 

        lines[hit_count] = $0;

        tapes[hit_count] = this_tape;
        startFilemarks[hit_count] = tape_filemark_num;
        endFilemarks[hit_count] = tape_filemark_num;
        startOffsets[hit_count] = 0;
        endOffsets[hit_count] = 0;
        filetype[hit_count] = "???";
        chunkSize[hit_count] = -1;
        endChunkSize[hit_count] = -1;
        startChan[hit_count] = -1;
        numChans[hit_count] = 0;
        name[hit_count] = "???";
        uuid[hit_count] = "???";
    }
}

# file list stuff
/<FileList>/ {
    filelist_flag = 1;
}

/<\/FileList>/ {
    filelist_flag = 0;
}

/<TapeFilemark/ {
    mask = 0;
    line = strip_whitespaces($0);
    tape_filemark_num = substr(line, 19, length(line) - 19);
} 

/<FirstFilename>/ {

    if (filelist_flag && (curFileTape == this_tape))
    {
        filename = getValue();

        if (shotname != "")
        {
            our_shotname = get_shotname_from_filename(filename);

            # handle awk fudging between expressions and constant regexps
            # e.g. shotname of 106+109_111_1 needs to be entered as
            # 106\\\\+109_111_1 for the shotname matching above to work
            # but a \\ has to be removed from when we compare it to 
            # our_shotname - annoying and confusing!
            temp_shotname = shotname;
            sub(/\\/, "", temp_shotname);
            good = 0;
	    if ( (ignorecase && (tolower(our_shotname)==tolower(temp_shotname))) \
                  || (our_shotname == temp_shotname) )
            {
                mask += name_hit;
                info_string = "shotname match (" our_shotname ")";
                hit(info_string);
            }
        }

        if (rollname != "")
        {
            our_rollname = get_rollname_from_filename(filename);
            if ( (ignorecase && (tolower(our_roll) == tolower(rollname))) || \
                 (our_roll == rollname) )
            {
                mask += roll_hit;
                info_string = "rollname match (" our_roll ")";
                hit(info_string);
            }
        }

        if (timecode1 != 0)
        {
            check_frames(frames_since_midnight1, timecode1, 
                         frames_since_midnight2, timecode2);
        }
    }
}

#######################################################################
# Shot list section parsing...

function convert_timecode_to_frames(timecode)
{
    split(timecode, timecode_comps, ":");
    frames = strtonum(timecode_comps[1]);
    frames *= 60;
    frames += strtonum(timecode_comps[2]);
    frames *= 60;
    frames += strtonum(timecode_comps[3]);
    frames *= fps;
    frames += strtonum(timecode_comps[4]);
    return frames;
}

function getValue()
{
    FS=">"; $0 = $0;
    end = index($2, "<") - 1;
    value = substr($2, 1, end);
    return value;
}

function dropZeroes(in_string)
{
    pos = 1;
    while (substr(in_string, pos, 1) == "0")
        pos += 1;
    out_string = substr(in_string, pos, length(in_string) - pos + 1);
    return out_string;
}

function printTidiedList(sorted, idx)
{
    # collate entries in contiguous groups: e.g.
    # 1, 5, 7, 6, 8, 2, 10 -> 1-2 5-8 10

    # sort list first
    asort(sorted);

    # now go through list and work out contiguous sections and
    # display as start and end values with dash in middle
    lastDisplayedStr = "";
    for (j=1; j in sorted; j++)
    {
        val = int(sorted[j]);
        if (!val)
           continue;
        lastVal = int(sorted[j-1]);

        str = dropZeroes(sorted[j]);
        lastStr = dropZeroes(sorted[j-1]);

        # if its a repeat of last entry skip it unless its the last number
        if ((str == lastStr) && (j != (idx-1)))
            continue;

        if (j == 1)
        {
            # always display first entry
            printf("%s", str, val);
            lastDisplayedStr = val;
        }
        else if (j == (idx-1))
        {
            # if this is the last value we need
            # to display something at the end of the
            # list

            # if there's no gap with the last value
            # then mark as the end of a list
            if (((val - lastVal) <= 1) && !index(lastStr, "end:"))
                printf("-%s", str);
            else 
            {
                # there is a gap between last value
                if (lastStr != lastDisplayedStr)
                    printf("-%s", lastStr);

                printf(" %s", str);
            }
        }
        else if (((val - lastVal) > 1) || index(lastStr, "end:"))
        {
            # we are mid list and there is a gap in
            # values between this one and the last

            # if we've not displayed the last value
            # cos it was the continuation of a sequence
            # then display it
            # (e.g. first entry in list and then a
            # gap)
            if (lastStr != lastDisplayedStr)
                printf("-%s", lastStr);

            # display this value
            printf(" %s", str);
            lastDisplayedStr = str;
        }
    }
    for (j in sorted)
        delete sorted[j];
}

# keep track of which tape this toc is for
/<ThisTape>/ {
    nextLineTape = 1;
}

/<\/ThisTape>/ {
    nextLineTape = 0;
}

/<Tape>/ {
    if (nextLineTape)
    {
        this_tape = getValue();
        nextLineTape = 0;
    }
    if (inShot)
        curSectionTape = getValue();
    if (filelist_flag)
        curFileTape = getValue();
}


# set scope for shot, roll and tc persistence
/<Shot id/ {
    inShot = 1;
    shot_name = "";
    roll_name = "";
    tc = "";
    mask = 0;
    curSectionTape = 0;
    curSectionStart = 0;
    curSectionEnd = 0;
    startFileIdx = 0;
    endFileIdx = 0;
    frameSpan = 0;
    thisChunkSize = 0;
    thisEndChunkSize = 0;
}

/<\/Shot>/ {
    if (inShot)
        inShot = 0;
    our_shot_uuid = "";
}

/<TapeSection/ {
   startChanIdx = index($1, "StartChan=");
   numChansIdx = index($1, "NumChans=");
   thisStartChan = substr($1, startChanIdx + 11, numChansIdx - startChanIdx - 13);
   thisNumChans = substr($1, numChansIdx + 10, length($1) - numChansIdx - 10);
}

/<\/TapeSection>/ {
    sameName = 1;
    if (1 in name)
    {
        if ((gensub(/ /, "_", "g", our_shotname) "-" our_shot_uuid) != name[1] "-" uuid[1])
            sameName = 0; 
    }
    if ((mask == hitmask) && (sameName || !firstHitOnly))
    {
        hit_count++;
        tapes[hit_count] = curSectionTape;
        if ( ((inChunkNum > 0) || (outChunkNum != -1)) && image_file_flag && !shotlevel)
        {
            startFilemarks[hit_count] = curSectionStart + inChunkNum;
            endFilemarks[hit_count] = curSectionStart + outChunkNum;
            startOffsets[hit_count] = startFileIdx;
            endOffsets[hit_count] = endFileIdx;
        }
        else
        {
            startFilemarks[hit_count] = curSectionStart;
            endFilemarks[hit_count] = curSectionEnd;
            startOffsets[hit_count] = 0;
            endOffsets[hit_count] = 0;
        }

        filetype[hit_count] = thisFiletype;
        chunkSize[hit_count] = image_file_flag ? thisChunkSize : 1;
        endChunkSize[hit_count] = image_file_flag ? thisEndChunkSize : 1;
        startChan[hit_count] = audio_file_flag ? thisStartChan : -1;
        numChans[hit_count] = audio_file_flag ? thisNumChans : -1;
        name[hit_count] = gensub(/ /, "_", "g", our_shotname);
        uuid[hit_count] = our_shot_uuid;
    }
}

function getFileTypeFromFileList()
{
    startIdx = index($0, "=")+2;
    endIdx = index($0, ">")-1;
    return substr($0, startIdx, endIdx-startIdx);
}

/<MetadataFileList/ {
    thisFiletype = getFileTypeFromFileList();
    metadata_file_flag = 1;
}

/<\/MetadataFileList/ {
    metadata_file_flag = 0;
}

/<AudioFileList/ {
    thisFiletype = getFileTypeFromFileList();
    audio_file_flag = 1;
}

/<\/AudioFileList/ {
    audio_file_flag = 0;
}

/<VideoFileList/ {
    thisFiletype = getFileTypeFromFileList();
    if (index($1, "dpx") || index($1, "ari"))
        image_file_flag = 1;
    else
        image_file_flag = 0;
}

/<\/VideoFileList/ {
    image_file_flag = 0;
}

# keep track of details
/<ShotName>/ { 
    our_shotname = getValue();
    if (length(shotname))
    {
        temp_shotname = shotname;
        sub(/\\/, "", temp_shotname);

        if (ignorecase && (tolower(our_shotname) == tolower(temp_shotname)))
            mask += name_hit;
        else if (our_shotname == temp_shotname)
            mask += name_hit;
    }
}

/<RollName>/ {
    if (length(rollname))
    {
        our_rollname = getValue();

        # If rollname has a space in it then replace with
        # underscore: spaces are illegal in CMX3600 EDLs
        # and avid puts in an underscore.
        # This may need reviewing/finessing.
        # CR! 07/10/11
        if (index(our_rollname, " ")>0)
            our_rollname=gensub(/ /, "_", "g", our_rollname);

        if (ignorecase && (tolower(our_rollname) == tolower(rollname)))
            mask += roll_hit;
        else if (our_rollname == rollname)
            mask += roll_hit;
    }
}

/<NumFrames>/ {
    frameSpan = getValue();
}

/<UUID>/ {
    our_shot_uuid = getValue();
}

/<StartTc>/ {
    tc = getValue();
    frames = convert_timecode_to_frames(tc);
    inChunkNum = 0;
    outChunkNum = -1;
    if (frames_since_midnight1)
    {
        if (frames_since_midnight2 != -1)
        {
           # work out overlap of this chunk's frame span and frames 
           # since midnight (i.e. timecode event searching for) span
           if (frames > frames_since_midnight1)
               start = frames;
           else
               start = frames_since_midnight1;

           if ((frames + frameSpan) < frames_since_midnight2)
               end = frames + frameSpan;
           else
               end = frames_since_midnight2;

           overlap = end - start;
           if (overlap > 0)
               mask += anytc_hit;
        }
        else
        {
            if ((frames_since_midnight1 >= frames) &&
                (frames_since_midnight1 <= (frames + frameSpan)))
               mask += anytc_hit;
        }
    }
}

/<StartFilemark>/ {
    curSectionStart = getValue();
}

/<EndFilemark>/ {
    curSectionEnd = getValue();
}

/<ChunkSize>/ {
    thisChunkSize = getValue();
    if (thisChunkSize && and(mask, anytc_hit))
    {
        if (handle > 0)
        {
            if ((frames_since_midnight1 - handle) > frames)
                startFrameDiff = frames_since_midnight1 - handle - frames;
            else
                startFrameDiff = 0;
        }
        else
            startFrameDiff = frames_since_midnight1 - frames;

        inChunkNum = int(startFrameDiff / thisChunkSize);

        thisEndChunkSize = frameSpan % thisChunkSize;
        if (!thisEndChunkSize)
            thisEndChunkSize = thisChunkSize;

        if (frames_since_midnight2 != -1)
        {
            # deduce which filemark end point is in
            if (handle > 0)
            {
                if (((frame_since_midnight2 + handle) - frames) < frameSpan)
                    endFrameDiff = frames_since_midnight2 + handle - frames;
                else
                    endFrameDiff = frameSpan;
            }
            else
                endFrameDiff = frames_since_midnight2 - frames;

            outChunkNum = int(endFrameDiff / thisChunkSize);

            # REVIEW: found on GOT offload that the num frames in the shot list
            # for a shot was one more than the actual number of dpx files which
            # buggered up the thisEndChunkSize count above - this is a band aid
            # for that but why did it happen in the first place?
            if (inChunkNum == outChunkNum)
                thisEndChunkSize = thisChunkSize;
            
            # deduce how many files into the filemark with first timecode in the
            # in point is
            startFileIdx = startFrameDiff % thisChunkSize;
            if (startFileIdx < 0)
                startFileIdx = 0;
            if (inChunkNum < 0)
                inChunkNum = 0;

            # deduce how many files after the filemark with the out point in are
            # left
            endChunkFileIdx = endFrameDiff % thisChunkSize;
            lastChunkNum = int(frameSpan / thisChunkSize) - 1;
            if (outChunkNum == lastChunkNum)
            {
                # if the out point is in the last filemark for this section of
                # material then it probably wont be normal chunksize - work out
                # what it is
                endFileIdx = thisEndChunkSize - endChunkFileIdx;
                if (endFileIdx < 0)
                    endFileIdx = 0;
            }
            else
            {
                if (outChunkNum > lastChunkNum)
                {
                    outChunkNum = lastChunkNum;
                    endFileIdx = 0;
                }
                else
                    endFileIdx = thisChunkSize - endChunkFileIdx;
             }
  
             if (endFileIdx == thisChunkSize)
                 mask = 0;
        }
        else
            outChunkNum = inChunkNum;
    }
}

END {
    i = 1;
    entriesDisplayed = 0;
    last_filemark_hit = -1;
    first_filemark_hit = -1;

    while ((entriesDisplayed < hit_count)&& (i<10000))
    {
        if (length(tapes[i]))
        {
            this_tape = tapes[i];
            idx=1;
            # find all entries for a tape
            for (j=1; j<=hit_count; j++)
            {
                # If verbose then we print out an line per filemark and include
                # extra info.
                # If non verbose then each line contains a contiguous stretch of
                # filemarks.
                if (!verbose && (j == 1))
                    printf("Tape: %s marks: ", this_tape);

                if (tapes[j] == this_tape)
                {
                    for (k=startFilemarks[j]+0; k<=endFilemarks[j]+0; k++)
                    {
                        if (verbose)
                            printf("Tape: %s marks: ", this_tape);

                        registered = 0;
                        if ((k == startFilemarks[j]) && startOffsets[j])
                        {
                            str = sprintf("%06d[begin:%d]", k,
                                              startOffsets[j]);
                            registered = 1;
                            if (!verbose)
                                sorted[idx++] = str;
                        }
                        if ((k == endFilemarks[j]) && endOffsets[j])
                        {
                            str = sprintf("%06d[end:%d]", k,
                                          endOffsets[j]);
                            registered = 1;
                            if (!verbose)
                                sorted[idx++] = str;
                        }

                        if (!registered)
                        {
                            str = sprintf("%06d", k);
                            if (!verbose)
                                sorted[idx++] = str;
                        }
                        
                        if (verbose)
                        {
                            str = dropZeroes(str);
                            if ((k == endFilemarks[j])) # && (endChunkSize[j] > 0))
                                str = str " chunksize: " endChunkSize[j];
                            else
                                str = str " chunksize: " chunkSize[j];
                            if (numChans[j] > 0)
                            {
                                str = str " startchan: " startChan[j];
                                str = str " numchans: " numChans[j];
                            }
                            str = str " shotname: " name[j] " uuid: " uuid[j] " filetype: " filetype[j];

                            printf("%s", str);
                            if (context)
                                printf(" (%s)", context);
                            printf("\n");
                        }
                    }
                    # clear name so don't check for this tape again
                    tapes[j] = "";
                    entriesDisplayed++;
                }
            }

            if (!verbose)
            {
                printTidiedList(sorted, idx);
                if (context)
                    printf(" (%s)", context);
                printf("\n");
            }
        }
        i++;
    }
    if (!hit_count)
        printf("\n");
}

